@@ -9,7 +9,8 @@ Data Ingestion into the Switch Automation
 
 Complete package for ingestion data into the Switch Automation Platform.
 """
-__all__ = ['dataset', 'analytics', 'pipeline', 'error_handlers', 'integration', 'initialize', 'cache']
+__all__ = ['dataset', 'analytics', 'pipeline', 'error_handlers', 'integration',
+           'initialize', 'cache', 'extensions', 'email', 'controls', '_compute', 'generate_filepath']
 
 from .initialize import initialize
 from . import dataset
@@ -19,6 +20,12 @@ from . import error_handlers
 # from . import platform_insights
 from . import integration
 from . import cache
+from . import extensions
+from . import email
+from . import controls
+from . import _compute
+from ._utils._utils import generate_filepath
+from . import _guide
 
 
 # import logging
@@ -33,4 +40,4 @@ from . import cache
 # log.addHandler(_ch)
 # https://dev.to/joaomcteixeira/setting-up-python-logging-for-a-library-app-6ml
 
-__version__ = "0.2.27"
+__version__ = "0.6.10"
