@@ -25,14 +25,14 @@ consoleHandler.setFormatter(formatter)
 
 
 _authority = {
-  "dev": {
-    "uitemplateid": "dev",
-    "base_url": "https://switchdevb2c.b2clogin.com/switchdevb2c.onmicrosoft.com",
-  },
-  "prod": {
-    "base_url": "https://switchautomation.b2clogin.com/switchautomation.onmicrosoft.com",
-    "uitemplateid": "default"
-  }
+    "dev": {
+        "uitemplateid": "dev",
+        "base_url": "https://switchdevb2c.b2clogin.com/switchdevb2c.onmicrosoft.com",
+    },
+    "prod": {
+        "base_url": "https://switchautomation.b2clogin.com/switchautomation.onmicrosoft.com",
+        "uitemplateid": "default"
+    }
 }
 
 
@@ -69,7 +69,7 @@ def _validate_credentials(creds: SwitchCredentials, version: str):
         return False
 
 
-def get_switch_credentials(environment: SWITCH_ENVIRONMENT, portfolio_id):
+def get_switch_credentials(environment: SWITCH_ENVIRONMENT, portfolio_id, port: int):
     """Read Credentials
 
     Reads credentials from credential store
@@ -80,6 +80,8 @@ def get_switch_credentials(environment: SWITCH_ENVIRONMENT, portfolio_id):
         Either development or production environment
     portfolio_id
         Get credentials for the given portfolio id
+    port: int
+        Port used for instantiating local server for authentication
 
     Returns
     -------
@@ -88,13 +90,17 @@ def get_switch_credentials(environment: SWITCH_ENVIRONMENT, portfolio_id):
 
     """
 
-    login_context = fetch_login_context(environment=environment, api_project_id=portfolio_id)[1]
+    login_context = fetch_login_context(
+        environment=environment, api_project_id=portfolio_id)[1]
 
     # Fetch from credentials store if it's already been cached
+    iot_endpoint = 'https://realtime-au.switchautomation.com/iot'
+    
     if login_context is not None:
         datacenter = login_context['dataCenter']
         key_prefix = f"{environment}_{datacenter}" if environment == 'Development' else datacenter
-        cur_credentials = read_credentials(key_prefix, login_context['portfolioId'], login_context['portfolioName'])
+        cur_credentials = read_credentials(
+            key_prefix, login_context['portfolioId'], login_context['portfolioName'])
         if _validate_credentials(cur_credentials, login_context['version']):
             return cur_credentials
     else:
@@ -104,29 +110,35 @@ def get_switch_credentials(environment: SWITCH_ENVIRONMENT, portfolio_id):
     # Credentials do not exist, take user through browser auth journey
     config_env = 'dev' if environment == 'Development' else 'prod'
     authority = f"{_authority[config_env]['base_url']}/{login_context['policyName']}"
-    app = CustomPublicClientApplication(login_context['applicationId'], authority=authority)
+    app = CustomPublicClientApplication(
+        login_context['applicationId'], authority=authority)
 
     uitemplateid = _authority[config_env]['uitemplateid']
     prompt = login_context['prompt'] if login_context['prompt'] != '' else None
-    auth_response = app.acquire_token_interactive_custom(scopes=[], prompt=prompt, port=51796,
+    auth_response = app.acquire_token_interactive_custom(scopes=[], prompt=prompt, port=port,
                                                          auth_params={
                                                              "uitemplateid": uitemplateid,
                                                              "portfolioId": portfolio_id,
                                                              "environment": environment
     })
 
+    if datacenter.lower() == 'centralus':
+        iot_endpoint = 'https://realtime-us.switchautomation.com/iot'
+
     id_token = auth_response['id_token']
 
     api_base_url = AUTH_ENDPOINT_DEV if environment == 'Development' else AUTH_ENDPOINT_PROD
-    _, profile = fetch_user_profile_details(api_base_url, id_token, environment, portfolio_id)
+    _, profile = fetch_user_profile_details(
+        api_base_url, id_token, environment, portfolio_id)
 
     if profile is None:
-        raise Exception('Unable to obtain User Profile. Please try again or contact your administrator.')
+        raise Exception(
+            'Unable to obtain User Profile. Please try again or contact your administrator.')
 
     credentials = SwitchCredentials(login_context['version'], profile['email'],
                                     profile['userId'], login_context['portfolioId'],
                                     login_context['portfolioName'], id_token, profile['apiEndpoint'],
-                                    environment, profile['subscriptionKey'])
+                                    environment, profile['subscriptionKey'], iot_endpoint)
 
     store_credentials(credentials, key_prefix=key_prefix)
 
@@ -162,14 +174,16 @@ def fetch_user_profile_details(api_base_url: str, token: str, environment: SWITC
 
     url = f"{api_base_url}/user-profile?portfolioId={api_project_id}&switchEnvironment={environment}"
 
-    response = requests.request("GET", url, data=payload, timeout=20, headers=headers)
+    response = requests.request(
+        "GET", url, data=payload, timeout=20, headers=headers)
     response_status = '{} {}'.format(response.status_code, response.reason)
     if response.status_code != 200:
         logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
                      response.reason)
         return response_status, None
     elif len(response.text) == 0:
-        logger.error('No data returned for this API call. %s', response.request.url)
+        logger.error('No data returned for this API call. %s',
+                     response.request.url)
         return response_status, None
     return response_status, response.json()
 
@@ -200,13 +214,15 @@ def fetch_login_context(environment: SWITCH_ENVIRONMENT, api_project_id: UUID):
 
     url = f"{base_url}/login-context?portfolioId={api_project_id}&switchEnvironment={environment}&type=PyPkg"
 
-    response = requests.request("GET", url, data=payload, timeout=20, headers=headers)
+    response = requests.request(
+        "GET", url, data=payload, timeout=20, headers=headers)
     response_status = '{} {}'.format(response.status_code, response.reason)
     if response.status_code != 200:
         logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
                      response.reason)
         return response_status, None
     elif len(response.text) == 0:
-        logger.error('No data returned for this API call. %s', response.request.url)
+        logger.error('No data returned for this API call. %s',
+                     response.request.url)
         return response_status, None
     return response_status, response.json()
