@@ -42,12 +42,14 @@ DATA_INGESTION_CONTAINER = Literal['data-ingestion-adx', 'data-ingestion-timeser
 
 QUERY_LANGUAGE = Literal['sql', 'kql']
 
+RESPONSE_TYPE = Literal['dataframe', 'json', 'string']
+
 ERROR_TYPE = Literal['DateTime', 'MissingDevices', 'NonNumeric', 'MissingRequiredField(s)', 'MissingSite(s)',
-                     'DuplicateRecords', 'MissingSensors', 'UnableToReadFile', 'InvalidFileType', 'InvalidInputSettings']
+                     'DuplicateRecords', 'MissingSensors', 'UnableToReadFile', 'InvalidFileType', 'InvalidInputSettings', 'ReadingSummariesTimeout']
 
 PROCESS_STATUS = Literal['ActionRequired', 'Failed']
 
-MAPPING_ENTITIES = Literal['Installations', 'Devices/Sensors', 'Readings', 'Work Orders']
+MAPPING_ENTITIES = Literal['Installations', 'Devices/Sensors', 'Readings', 'Work Orders', 'IQ']
 
 WORK_ORDER_CATEGORY = Literal['Preventative Maintenance', 'Tenant Request']
 
@@ -56,6 +58,10 @@ WORK_ORDER_PRIORITY = Literal['Low', 'Medium', 'High']
 WORK_ORDER_STATUS = Literal['Submitted', 'Open', 'In Progress', 'Waiting for 3rd Party', 'Resolved', 'Abandoned',
                             'Closed']
 
+RESERVATION_STATUS = Literal['Booked', 'In Progress', 'Complete', 'Cancelled']
+
+RESOURCE_TYPE = Literal['MeetingRoom', 'ConferenceCentre', 'Desk', 'Other']
+
 QUEUE_NAME = Literal['task', 'highpriority']
 
 SCHEDULE_TIMEZONE = Literal['Local', 'Utc']
@@ -70,6 +76,21 @@ INTEGRATION_SETTINGS_EDITORS = Literal['text_box', 'numeric_stepper', 'custom_co
 CACHE_SCOPE = Literal['Portfolio', 'Task', 'DataFeed']
 TAG_LEVEL = Literal['Site', 'Device', 'Sensor']
 
-DATA_SET_QUERY_PARAMETER_TYPES = Literal['String', 'Number', 'DateTime', 'Keyword', 'Boolean', 'NumberArray', 'StringArray']
+DATA_SET_QUERY_PARAMETER_TYPES = Literal['String', 'Number', 'DateTime', 'Keyword', 'Boolean', 'NumberArray',
+                                         'StringArray']
 
 ADX_TABLE_DEF_TYPES = Literal['object', 'dynamic', 'int32', 'int64', 'float32', 'float64', 'datetime', 'string']
+
+TASK_PRIORITY = Literal['default', 'standard', 'advanced']
+
+TASK_FRAMEWORK = Literal['PythonScriptFramework', 'TaskInsightsEngine']
+
+AMORTISATION_METHOD = Literal['Exclusive', 'Inclusive']
+
+PERFORMANCE_STATISTIC_METRIC_SYSTEM = Literal['metric', 'imperial']
+
+GUIDES_EXTERNAL_TYPES = Literal['SwitchGuides']
+
+GUIDES_SCOPE = Literal['Portfolio-wide']
+
+IMPORT_STATUS = Literal['New', 'ToImport', 'OnHold', 'Exclude', 'Edited', 'ToRemove']
