@@ -7,7 +7,9 @@
 A module for .....
 """
 # import datetime
+import os
 import re
+import tempfile
 import uuid
 from collections import namedtuple
 from typing import Union, List
@@ -19,10 +21,10 @@ import logging
 import sys
 import requests
 from requests.adapters import HTTPAdapter
-
 from requests.sessions import session
 from urllib3.util.retry import Retry
-from .._utils._constants import WORK_ORDER_STATUS, WORK_ORDER_CATEGORY, WORK_ORDER_PRIORITY
+from .._utils._constants import (WORK_ORDER_STATUS, WORK_ORDER_CATEGORY, WORK_ORDER_PRIORITY,
+                                 PERFORMANCE_STATISTIC_METRIC_SYSTEM, RESERVATION_STATUS, RESOURCE_TYPE)
 
 __all__ = ['generate_password', 'convert_to_sqm', 'convert_to_pascal_case', 'ValidatedSetterProperty', 'ApiInputs',
            'DiscoveryIntegrationInput', 'DataFeedFileProcessOutput', 'requests_retry_session', 'is_valid_uuid', 'convert_bytes']
@@ -50,7 +52,8 @@ class ValidatedSetterProperty:
 
 
 def _column_name_cap(columns) -> list:
-    renamed_columns = [name[0].upper() + name[1:] for name in columns.to_list()]
+    renamed_columns = [name[0].upper() + name[1:]
+                       for name in columns.to_list()]
     return renamed_columns
 
 
@@ -88,7 +91,9 @@ ApiHeaders = namedtuple('ApiHeaders', ['default', 'integration'])
 
 ApiInputs = namedtuple('ApiInputs',
                        ['email_address', 'user_id', 'api_project_id', 'data_feed_id', 'data_feed_file_status_id',
-                        'bearer_token', 'api_base_url', 'api_projects_endpoint', 'subscription_key', 'api_headers'])
+                        'bearer_token', 'api_base_url', 'api_projects_endpoint', 'subscription_key', 'api_headers', 'iot_url'])
+
+IngestionMode = namedtuple('IngestionMode', ['Queue', 'Stream'])
 
 DiscoveryIntegrationInput = namedtuple(
     'DiscoveryIntegrationInput',
@@ -187,7 +192,8 @@ def convert_to_pascal_case(text: Union[str, List[str]]):
         text = str.upper(text[0]) + text[1:]
         if not text:
             return text
-        text = re.sub(r"[^a-zA-Z0-9]([a-zA-Z0-9])", lambda matched: str.upper(matched.group(1)), text)
+        text = re.sub(r"[^a-zA-Z0-9]([a-zA-Z0-9])",
+                      lambda matched: str.upper(matched.group(1)), text)
         return re.sub(r"[^a-zA-Z0-9]", '', text)
     elif type(text) == list and {isinstance(text[j], str) for j in range(len(text))}.issubset([True]):
         for i in range(len(text)):
@@ -197,7 +203,8 @@ def convert_to_pascal_case(text: Union[str, List[str]]):
             text_item = str.upper(text_item[0]) + text_item[1:]
             if not text_item:
                 text[i] = text_item
-            text_item = re.sub(r"[^a-zA-Z0-9]([a-zA-Z0-9])", lambda matched: str.upper(matched.group(1)), text_item)
+            text_item = re.sub(r"[^a-zA-Z0-9]([a-zA-Z0-9])",
+                               lambda matched: str.upper(matched.group(1)), text_item)
 
             text[i] = re.sub(r"[^a-zA-Z0-9]", '', text_item)
         return text
@@ -246,15 +253,16 @@ def requests_retry_session(retries=5, status_forcelist=[429, 500, 502, 503, 504]
     session.mount('https://', adapter=adapter)
     return session
 
+
 def is_valid_uuid(uuid_to_test: Union[uuid.UUID, str], version=4) -> bool:
     """
     Check if uuid_to_test is a valid UUID.
-    
+
      Parameters
     ----------
     uuid_to_test : str
     version : {1, 2, 3, 4}
-    
+
      Returns
     -------
     `True` if uuid_to_test is a valid UUID, otherwise `False`.
@@ -272,28 +280,68 @@ def is_valid_uuid(uuid_to_test: Union[uuid.UUID, str], version=4) -> bool:
 
             if uuid_to_test == uuid.UUID(int=0):
                 return False
-            
+
             return uuid_to_test.version == version
-         
+
         uuid_obj = uuid.UUID(str(uuid_to_test), version=version)
     except Exception:
         return False
 
     return str(uuid_obj) == uuid_to_test
 
+
+def generate_filepath(api_inputs: ApiInputs, filename: str, unique: bool = True):
+    """
+    Generates a filepath given the filename. Optionally, setting unique will append uniqueness to the filepath.
+
+    Parameters
+    ----------
+    filename: str
+        Name of the file to be generated. Include file extension where applicable.
+    unique: bool
+        If True, 
+        and api_inputs.data_feed_id and api_inputs.data_feed_file_status_id are valid identifiers,
+        the filepath will include the ids. Otherwise, the filepath will generate a unique directory prefix.
+        If False, the filepath will match the parameters and uniqueness is left to the user.
+
+    Returns
+    -------
+    str
+        Filepath of the generated file.
+    """
+
+    tempDirPath = tempfile.gettempdir()
+
+    directory = os.path.join(
+        tempDirPath, 'SwitchTaskInsights', 'projects', api_inputs.api_project_id)
+
+    if unique:
+        if is_valid_uuid(api_inputs.data_feed_id) and is_valid_uuid(api_inputs.data_feed_file_status_id):
+            directory = os.path.join(
+                directory, 'df', api_inputs.data_feed_id, 'dffs', api_inputs.data_feed_file_status_id)
+        else:
+            directory = os.path.join(directory, 'unq', str(uuid.uuid4()))
+
+    if not os.path.exists(directory):
+        os.makedirs(directory)
+
+    return os.path.join(directory, filename)
+
+
 def convert_bytes(bytes_number):
-    tags = [ "Byte", "Kilobyte", "Megabyte", "Gigabyte", "Terabyte" ]
- 
+    tags = ["Byte", "Kilobyte", "Megabyte", "Gigabyte", "Terabyte"]
+
     i = 0
     double_bytes = bytes_number
- 
-    while (i < len(tags) and  bytes_number >= 1024):
-            double_bytes = bytes_number / 1024.0
-            i = i + 1
-            bytes_number = bytes_number / 1024
- 
+
+    while (i < len(tags) and bytes_number >= 1024):
+        double_bytes = bytes_number / 1024.0
+        i = i + 1
+        bytes_number = bytes_number / 1024
+
     return str(round(double_bytes, 2)) + " " + tags[i]
 
+
 _site_schema = pandera.DataFrameSchema(
     {
         'InstallationName': pandera.Column(pandera.String, checks=pandera.Check.str_length(1, 255)),
@@ -312,6 +360,31 @@ _site_schema = pandera.DataFrameSchema(
     }
 )
 
+_performance_statistics_schema = pandera.DataFrameSchema(
+    columns={
+        'InstallationId': pandera.Column(pandera.String, checks=[
+            pandera.Check(lambda x: uuid.UUID(x).__class__ == uuid.UUID, element_wise=True)], required=True),
+        'Investment': pandera.Column(pandera.Float, checks=[pandera.Check.greater_than_or_equal_to(0)], required=True),
+        'RoiYears': pandera.Column(pandera.Int, checks=[pandera.Check.greater_than_or_equal_to(0)], required=True),
+        'CostSaving': pandera.Column(pandera.Float, required=True),
+        'Cost': pandera.Column(pandera.Float, checks=[pandera.Check.greater_than_or_equal_to(0)], required=True),
+        'ComparisonCost': pandera.Column(pandera.Float, checks=[pandera.Check.greater_than_or_equal_to(0)], required=True),
+        'ConsumptionSaving': pandera.Column(pandera.Float, required=True),
+        'Consumption': pandera.Column(pandera.Float, checks=[pandera.Check.greater_than_or_equal_to(0)], required=True),
+        'CarbonSaving': pandera.Column(pandera.Float, required=True),
+        'Carbon': pandera.Column(pandera.Float, checks=[pandera.Check.greater_than_or_equal_to(0)], required=True),
+        'ComparisonConsumption': pandera.Column(pandera.Float, checks=[pandera.Check.greater_than_or_equal_to(0)], required=True),
+        'ComparisonCarbon': pandera.Column(pandera.Float, checks=[pandera.Check.greater_than_or_equal_to(0)], required=True),
+        'ConsumptionUnit': pandera.Column(pandera.String, checks=pandera.Check.str_length(1, 255), required=True),
+        'MetricSystem': pandera.Column(pandera.String, checks=[
+            pandera.Check.isin(list(PERFORMANCE_STATISTIC_METRIC_SYSTEM.__args__), ignore_na=False)], required=True),
+    },
+    coerce=True,
+    strict=False,
+    name=None
+)
+
+
 _work_order_schema = pandera.DataFrameSchema(
     columns={
         'WorkOrderId': pandera.Column(pandera.String, required=True),
@@ -355,3 +428,96 @@ _work_order_schema = pandera.DataFrameSchema(
     strict=False,
     name=None
 )
+
+_reservation_schema = pandera.DataFrameSchema(
+    columns={
+        'ReservationId': pandera.Column(pandera.String, required=True),
+        'InstallationId': pandera.Column(pandera.String, checks=[
+            pandera.Check(lambda x: uuid.UUID(x).__class__ == uuid.UUID, element_wise=True)], required=True),
+        'ReservationSiteIdentifier': pandera.Column(pandera.STRING, coerce=True, required=True),
+        'Status': pandera.Column(pandera.String, checks=[
+            pandera.Check.isin(list(RESERVATION_STATUS.__args__), ignore_na=False)], required=True),
+        'RawStatus': pandera.Column(pandera.String, required=True),
+        'ReservationStart': pandera.Column(pandera.DateTime, required=True),
+        'ReservationEnd': pandera.Column(pandera.DateTime, required=True),
+        'CreatedDate': pandera.Column(pandera.DateTime, required=True),
+        'LastModifiedDate': pandera.Column(pandera.DateTime, required=True, nullable=True),
+        'ObjectPropertyId': pandera.Column(pandera.String, checks=[
+            pandera.Check(lambda x: uuid.UUID(x).__class__ == uuid.UUID, element_wise=True)], required=True),
+        'ResourceType': pandera.Column(pandera.String, checks=[
+            pandera.Check.isin(list(RESOURCE_TYPE.__args__), ignore_na=False)], required=True),
+        'RawResourceType': pandera.Column(pandera.String, required=True, nullable=True),
+        'ReservationSystem': pandera.Column(pandera.String, required=True, nullable=True),
+        'DataFeedFileStatusId': pandera.Column(pandera.String, checks=[
+            pandera.Check(lambda x: uuid.UUID(x).__class__ == uuid.UUID, element_wise=True)], required=True),
+        'ReservationName': pandera.Column(pandera.String, required=False, nullable=True),
+        'Description': pandera.Column(pandera.String, required=False, nullable=True),
+        'ReservedById': pandera.Column(pandera.String, required=False, nullable=True),
+        'ReservedByEmail': pandera.Column(pandera.String, required=False, nullable=True),
+        'LocationId': pandera.Column(pandera.String, required=False, nullable=True),
+        'RawLocationId': pandera.Column(pandera.String, required=False, nullable=True),
+        'Location': pandera.Column(pandera.String, required=False, nullable=True),
+        'RawLocation': pandera.Column(pandera.String, required=False, nullable=True),
+        'Source': pandera.Column(pandera.String, required=False, nullable=True),
+        'AttendeeCount': pandera.Column(pandera.Int, required=False, nullable=True),
+    },
+    coerce=True,
+    strict=False,
+    name=None
+)
+
+
+def requests_retry_session2(retries: int = 5, status_forcelist: List[int] = [408, 429, 500, 502, 503, 504],
+                            backoff_factor=2, session: Union[requests.Session, None] = None,
+                            allowed_methods=["GET", "POST"], raise_on_status: bool = False):
+    """
+    Provide retry policy for requests fired.
+
+    Parameters
+    ----------
+    retries : int
+        Number of retries performed before returning last retry instance's response status.
+        Defaults to 5.
+    status_forcelist : List[number]
+        A set of integer HTTP status codes that we should force a retry on.
+        Defaults to 429, 500, 502, 503, 504.
+    backoff_factor : number
+        A backoff factor to apply between attempts after the second try (most errors are resolved immediately by a
+        second try without a delay).
+        {backoff factor} * (2 ** ({number of total retries} - 1)) seconds
+        Defaults to 2
+    session : Union[requests.Session, None]
+        Either none or a requests.Session() object.
+    allowed_methods : List[string]
+        Set of uppercased HTTP method verbs that we should retry on
+        Defaults to: GET
+    raise_on_status: bool, optional
+        defaults to False which means no exception is raised on final retry but response status is returned. If True,
+        then exception is raised instead of providing a response.
+
+    Returns
+    -------
+        Session containing the retry configuration.
+        Can be changed as requests_retry_session().get() or .post() depending on requirements
+    """
+
+    if retries > 10:
+        logger.warning(f"Max retries allowed is 10. ")
+        retries = 10
+
+    requests_retry_session2.retries = retries
+    session = session or requests.Session()
+    retry = Retry(
+        total=retries,
+        read=retries,
+        connect=retries,
+        status_forcelist=status_forcelist,
+        backoff_factor=backoff_factor,
+        allowed_methods=allowed_methods,
+        raise_on_status=raise_on_status
+    )
+    adapter = HTTPAdapter(max_retries=retry)
+    session.mount('http://', adapter=adapter)
+    session.mount('https://', adapter=adapter)
+    session.retries = retries
+    return session
