@@ -7,6 +7,6 @@
 A module for .....
 """
 
-from .analytics import (get_parent_modules_list, get_clone_modules_list, sensor_has_not_changed)  ## , run_clone_modules
+from .analytics import (upsert_performance_statistics, get_parent_modules_list, get_clone_modules_list, sensor_has_not_changed)  ## , run_clone_modules
 
-__all__ = ['get_parent_modules_list', 'get_clone_modules_list', 'sensor_has_not_changed']  # 'run_clone_modules',
+__all__ = ['upsert_performance_statistics', 'get_parent_modules_list', 'get_clone_modules_list', 'sensor_has_not_changed']  # 'run_clone_modules',
