@@ -6,6 +6,7 @@
 import sys
 import uuid
 import logging
+
 from ._authentication._authentication import get_switch_credentials
 from ._utils._utils import ApiInputs, ApiHeaders
 from ._utils._constants import SWITCH_ENVIRONMENT
@@ -22,7 +23,7 @@ formatter = logging.Formatter('%(asctime)s  switch_api.%(module)s.%(funcName)s
 consoleHandler.setFormatter(formatter)
 
 
-def initialize(api_project_id: uuid.UUID, environment: SWITCH_ENVIRONMENT = 'Production') -> ApiInputs:
+def initialize(api_project_id: uuid.UUID, environment: SWITCH_ENVIRONMENT = 'Production', custom_port: int = 51796) -> ApiInputs:
     """
     Initialize session with API when running scripts on your local machine.
 
@@ -32,20 +33,22 @@ def initialize(api_project_id: uuid.UUID, environment: SWITCH_ENVIRONMENT = 'Pro
         The ApiProjectID for the portfolio.
     environment: SWITCH_ENVIRONMENT
         (Optional) The Switch API environment to interact with
+    custom_port: int
+        (Optional) Custom network port to initialize instance of the server when authenticating
 
     Returns
     -------
     ApiInputs
         Returns the ApiInputs namedtuple that is required as an input to other functions.
     """
-
     try:
         uuid.UUID(api_project_id)
     except:
-        sw.logger.error(f"Invalid value provided '{api_project_id}'- please provide a valid api_project_id. ")
+        logger.error(
+            f"Invalid value provided '{api_project_id}'- please provide a valid api_project_id. ")
         return False
 
-    creds = get_switch_credentials(environment, api_project_id)
+    creds = get_switch_credentials(environment, api_project_id, custom_port)
 
     logger.info("Successfully initialized.")
     api_base_url = f"https://{creds.api_endpoint}/api/1.0"
@@ -81,4 +84,4 @@ def initialize(api_project_id: uuid.UUID, environment: SWITCH_ENVIRONMENT = 'Pro
     return ApiInputs(email_address=creds.email, user_id=creds.user_id, api_project_id=api_project_id,
                      data_feed_id=data_feed_id, data_feed_file_status_id=data_feed_file_status_id, bearer_token=token,
                      api_base_url=api_base_url, api_projects_endpoint=api_projects_endpoint,
-                     subscription_key=creds.subscription_key, api_headers=api_headers)
+                     subscription_key=creds.subscription_key, api_headers=api_headers, iot_url=creds.iot_endpoint)
