@@ -13,6 +13,7 @@ import requests
 import datetime
 import logging
 import sys
+import re
 from .._utils._constants import SUPPORT_PAYLOAD_TYPE  # , QUERY_LANGUAGE
 from .._utils._utils import ApiInputs, requests_retry_session, is_valid_uuid
 
@@ -176,20 +177,19 @@ def _timezone_dst_offsets(api_inputs: ApiInputs, date_from: datetime.date, date_
     try:
         response = requests_retry_session(method_whitelist=['POST']).post(url, json=payload, headers=headers)
 
-        response_status = '{} {}'.format(response.status_code, response.reason)
         if response.status_code != 200:
             logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
                          response.reason)
-            return response_status, pandas.DataFrame()
+            return pandas.DataFrame()
         elif len(response.text) == 0:
             logger.error('No data returned for this API call. %s', response.request.url)
-            return response_status, pandas.DataFrame()
+            return pandas.DataFrame()
 
         timezone_intervals = json.loads(response.text)
 
-        if not set(['installationTimezoneList','dstTransitions', 'dstIntervals']).issubset(timezone_intervals.columns):
+        if not timezone_intervals or ('dstTransitions' in timezone_intervals and len(timezone_intervals['dstTransitions']) == 0):
             logger.error('No Timezone DST Intervals returned for this API call. %s', response.request.url)
-            return response_status, pandas.DataFrame()
+            return pandas.DataFrame()
 
         site_timezones = pandas.DataFrame.from_dict(timezone_intervals['installationTimezoneList'])
         timezone_offsets = pandas.DataFrame.from_dict(timezone_intervals['dstTransitions'])
@@ -295,3 +295,53 @@ def _adx_support(api_inputs: ApiInputs, payload_type: SUPPORT_PAYLOAD_TYPE):
     logger.info("API Call successful.")
 
     return response_status, response.text
+
+
+def _extract_sql_credentials(cs: str) -> tuple:
+    """
+        Extract SQL Credentials from string
+
+        Parameters
+        ----------
+        cs : str
+            String containing sql credentials to be parsed.
+
+        Returns
+        -------
+        sql_server_name, sql_server_user_name, sql_server_password : tuple
+            Tuple containing SQL Server Name, UserName & Pwd
+
+        """
+    # Future: More elegant extraction.
+    try:
+        sql_server_name = re.search(r'Data Source=([a-zA-Z0-9.]+)\;', cs).group().replace('Data Source=', '')
+        sql_server_user_name = re.search(r'User ID=([a-zA-Z0-9.]+)\;', cs).group().replace('User ID=', '')
+        sql_server_password = re.search(r'Password=([a-zA-Z0-9@$#%^&*().]+)\;', cs).group().replace('Password=', '')
+    except AttributeError:
+        sql_server_name = None
+        sql_server_user_name = None
+        sql_server_password = None
+
+    return sql_server_name, sql_server_user_name, sql_server_password
+
+
+def _get_sql_connection_string(api_inputs: ApiInputs) -> str:
+    """
+        Get SQL Connection String
+
+        Parameters
+        ----------
+        api_inputs : ApiInputs
+            Object returned by initialize() function.
+
+        Returns
+        -------
+        connection_string: str
+            String containing sql connection string.
+
+        """
+    url = f"{api_inputs.api_projects_endpoint}/{api_inputs.api_project_id}/data-ingestion/sql-read-connection"
+    headers = api_inputs.api_headers.default
+    response = requests.get(url, headers=headers)
+    connection_string = response.text
+    return connection_string
