@@ -22,16 +22,19 @@ import logging
 import sys
 import uuid
 from io import StringIO
-from azure.servicebus import ServiceBusClient, ServiceBusReceiveMode  # , ServiceBusMessage
+from azure.servicebus import ServiceBusClient, ServiceBusReceiveMode
 from .._utils._utils import _is_valid_regex, ApiInputs, _column_name_cap, DataFeedFileProcessOutput
+from .._utils._marketplace import add_marketplace_item
 from .._utils._constants import (argus_prefix, EXPECTED_DELIVERY, MAPPING_ENTITIES,
-                                 QUEUE_NAME, ERROR_TYPE, SCHEDULE_TIMEZONE, DEPLOY_TYPE)
-from .._utils._platform import _get_ingestion_service_bus_connection_string
+                                 QUEUE_NAME, ERROR_TYPE, SCHEDULE_TIMEZONE, DEPLOY_TYPE, TASK_PRIORITY, TASK_FRAMEWORK, GUIDES_EXTERNAL_TYPES, GUIDES_SCOPE)
+from .._utils._platform import _get_ingestion_service_bus_connection_string, Blob
 from .pipeline import (Task, QueueTask, IntegrationTask, LogicModuleTask, AnalyticsTask, EventWorkOrderTask,
-                       DiscoverableIntegrationTask)
+                       DiscoverableIntegrationTask, BlobTask, Guide, IQTask)
+from ..extensions import ExtensionTask, replace_extensions_imports, has_extensions_support
 from .definitions import (IntegrationDeviceDefinition, EventWorkOrderFieldDefinition, AnalyticsSettings,
                           IntegrationSettings, IntegrationDeviceConfigPropertyDefinition)
 
+
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
 consoleHandler = logging.StreamHandler(sys.stdout)
@@ -73,8 +76,7 @@ class Automation:
             return pandas.DataFrame()
 
         if not issubclass(type(task), QueueTask):
-            logger.error(
-                "Driver must be an implementation of the QueueTask (Task).")
+            logger.error("Driver must be an implementation of the QueueTask (Task).")
             return False
 
         logger.info("Running queue for  %s (%s) using queue name: %s", str(type(task).__name__), str(task.id),
@@ -158,7 +160,8 @@ class Automation:
 
     @staticmethod
     def register_task(api_inputs: ApiInputs, task: Union[Task, IntegrationTask, DiscoverableIntegrationTask, QueueTask,
-                                                         AnalyticsTask, LogicModuleTask, EventWorkOrderTask]):
+                                                         AnalyticsTask, LogicModuleTask, EventWorkOrderTask, IQTask,
+                                                         ExtensionTask]):
         """Register the task.
 
         Registers the task that was defined.
@@ -169,8 +172,8 @@ class Automation:
             Object returned by initialize() function.
         task : Union[Task, IntegrationTask, DiscoverableIntegrationTask, QueueTask, AnalyticsTask, LogicModuleTask, EventWorkOrderTask]
             An instance of the custom class created from the Abstract Base Class `Task` or its abstract sub-classes:
-            `IntegrationTask`,`DiscoverableIntegrationTask`, `AnalyticsTask`, `QueueTask`, `EventWorkOrderTask`, or
-            `LogicModuleTask`.
+            `IntegrationTask`,`DiscoverableIntegrationTask`, `AnalyticsTask`, `QueueTask`, `EventWorkOrderTask`, `IQTask`,
+            `LogicModuleTask`, or `ExtensionTask`.
 
         Returns
         -------
@@ -178,33 +181,37 @@ class Automation:
 
         """
 
-        if not issubclass(type(task), Task):
-            logger.error(
-                "Driver must be an implementation of the Abstract Base Class (Task).")
+        if not issubclass(type(task), Task) and not issubclass(type(task), ExtensionTask):
+            logger.error("Driver must be an implementation of the Abstract Base Class (Task or ExtensionTask).")
             return False
 
         base_class = ''
-        allowed_base_classes = tuple([Task] + Task.__subclasses__())
+        allowed_base_classes = tuple([Task] + Task.__subclasses__() + [ExtensionTask])
         if issubclass(type(task), allowed_base_classes):
-            base_class = task.__class__.__base__.__qualname__
+            if len(task.__class__.__bases__) == 1:
+                base_class = task.__class__.__base__.__qualname__
+            else:
+                base_classes = []
+                for i in range(len(task.__class__.__bases__)):
+                    base_classes.append(task.__class__.__bases__[i].__qualname__)
+                base_class = list(set(base_classes).symmetric_difference([Guide.__qualname__]))
+                base_class = base_class[0]
 
         if base_class == '':
-            logger.error(
-                'Task must be an implementation of one of the Task sub-classes: IntegrationTask, LogicModuleTask, '
-                'AnalyticsTask, EventWorkOrderTask, DiscoverableIntegrationTask.')
+            logger.error(f'Task must be an implementation of one of the Task sub-classes: {Task.__subclasses__()}')
             return pandas.DataFrame()
         elif not set([base_class]).issubset(
                 {allowed_base_classes[i].__qualname__ for i in range(len(allowed_base_classes))}):
-            logger.error(f"Task must be an implementation of one of the Task sub-classes: IntegrationTask, "
-                         f"LogicModuleTask, AnalyticsTask, EventWorkOrderTask, DiscoverableIntegrationTask.)")
+            logger.error(f"Task must be an implementation of one of the Task sub-classes:  {Task.__subclasses__()}")
 
-        if type(task.mapping_entities) != list:
-            logger.error('The mapping_entities parameter must have type = list. ')
-            return pandas.DataFrame()
-        elif not set(task.mapping_entities).issubset(set(MAPPING_ENTITIES.__args__)):
-            logger.error('mapping_entities property must be a list containing one of the allowed values defined by the '
-                         'MAPPING_ENTITIES literal: %s', MAPPING_ENTITIES.__args__)
-            return pandas.DataFrame()
+        if not issubclass(type(task), ExtensionTask):
+            if type(task.mapping_entities) != list:
+                logger.error('The mapping_entities parameter must have type = list. ')
+                return pandas.DataFrame()
+            elif not set(task.mapping_entities).issubset(set(MAPPING_ENTITIES.__args__)):
+                logger.error('mapping_entities property must be a list containing one of the allowed values defined by the '
+                             'MAPPING_ENTITIES literal: %s', MAPPING_ENTITIES.__args__)
+                return pandas.DataFrame()
 
         task_settings = dict()
 
@@ -218,7 +225,7 @@ class Automation:
                 props = task.integration_device_type_definition.config_properties
 
                 if type(task.integration_device_type_definition.config_properties) != list:
-                    logger.error(f"The task.integration_device_type_definition.config_properties must have type = list")
+                    logger.error("The task.integration_device_type_definition.config_properties must have type = list")
                     return pandas.DataFrame()
                 else:
                     integration_settings_list = list()
@@ -299,7 +306,7 @@ class Automation:
         if base_class == 'AnalyticsTask':
             # Validate that the analytics_settings_definition matches required definition format.
             if type(task.analytics_settings_definition) != list:
-                logger.error(f"The task.analytics_settings_definition property must have type list")
+                logger.error("The task.analytics_settings_definition property must have type list")
                 return pandas.DataFrame
             else:
                 prop_types = {True: [], False: []}
@@ -322,7 +329,7 @@ class Automation:
             if len(analytics_settings_list) > 0:
                 task_settings['analytics_settings'] = analytics_settings_list
 
-        logger.info("Registering %s (%s) to the Switch Driver Library: ", str(type(task).__name__), str(task.id))
+        logger.info(f'Registering {str(type(task).__name__)} ({task.id}) to the Switch Driver Library:')
 
         json_payload_verify = {
             "driverId": str(task.id),
@@ -345,15 +352,18 @@ class Automation:
                          response.reason)
             return response_status, pandas.DataFrame()
         elif response.text == "Failed":
-            logger.error("This Driver Guid is already associated with a Driver with a Different Name.")
+            logger.error(f"There is already either a Driver Name called {type(task).__name__} or an existing Driver ID = {task.id}")
             return response_status, pandas.DataFrame()
 
+        script_code = replace_extensions_imports(task, 'extensions') if has_extensions_support(
+            task) else Automation._get_task_code(task)
+
         json_payload = {
             "driverId": str(task.id),
             "name": type(task).__name__,
             "specification": task.description,
-            "mappingEntities": task.mapping_entities,
-            "scriptCode": Automation._get_task_code(task),
+            "mappingEntities": task.mapping_entities if hasattr(task, 'mapping_entities') else [],
+            "scriptCode": script_code,
             "baseClass": base_class,
             "settings": task_settings
             # {"integration_settings":[{"property_name":"Dummy1", "display_name":"", "default_value":"", "allowed_values":[], "editor":""}], "work_order_input":[], "analytics_settings":[], }
@@ -377,6 +387,130 @@ class Automation:
 
         return df
 
+    @staticmethod
+    def register_guide_task(api_inputs: ApiInputs, task, local_folder_path: str,
+                            external_type: GUIDES_EXTERNAL_TYPES = 'SwitchGuides',
+                            scope: GUIDES_SCOPE = 'Portfolio-wide'):
+        """Register Guide Task
+
+        Registers the task that was defined and registers the guide and associated files to the marketplace.
+
+        1. Registers the driver to Script Drivers Table
+        2. Upload Blob Files from local folder path
+        3. Register driver to Marketplace Items
+
+        Parameters
+        ----------
+        api_inputs : ApiInputs
+            Object returned by initialize() function.
+        task : Union[DiscoverableIntegrationTask, GuideTask]
+            An instance of the custom class created from the Abstract Base Class `Task` or its abstract sub-classes:
+            `DiscoverableIntegrationTask`, or `GuideTask`.
+        local_folder_path : str
+            Local file path to the `.vue` and `.js` form files to be uploaded to blob
+        external_type : GUIDES_EXTERNAL_TYPES, optional
+            Defaults to 'SwitchGuides'.
+        scope : GUIDES_SCOPE, optional
+            Marketplace item scope. Defaults to 'Portfolio-wide'.
+
+        Returns
+        -------
+        Union[bool, pandas.DataFrame, tuple]
+
+        """
+
+        if not api_inputs or not api_inputs.api_base_url or not api_inputs.bearer_token:
+            logger.error("You must call initialize() before using the API.")
+            return pandas.DataFrame()
+
+        if len(task.__class__.__bases__) == 1:
+            logger.error(f"The task must subclass Task or one of its subclasses as well as the Guide class")
+            return False
+        elif issubclass(type(task), Guide) != True:
+            logger.error(f"The task must subclass the Guide class in addition to one of the Task sub-classes. ")
+            return False
+
+        min_tags = task.minimum_tags_met()
+        if min_tags != True:
+            if min_tags == False:
+                logger.error(f"The 'guide_tags' dictionary must not be empty. ")
+            logger.error("The 'guide_tags' must be a dictionary with a minimum of one key. ")
+            return False
+
+        short_desc_length = task.check_short_desc_length()
+        if short_desc_length != True:
+            if short_desc_length == False:
+                logger.error("The 'guide_short_description' must be a string that does not exceed 250 characters. ")
+                return False
+            logger.error(f"The 'guide_short_description' must be a string that does not exceed 250 characters. Current "
+                         f"length: {short_desc_length}")
+            return False
+
+
+        logger.info("Registering task...")
+        response = Automation.register_task(api_inputs, task)
+
+        # make it so this prints back properly
+        pandas.set_option('display.max_columns', 10)
+
+        _, register = (0, response) if isinstance(response, pandas.DataFrame) else response
+
+
+        if response.shape[0] == 0:
+            logger.error("Task registration failed. The guide forms were not uploaded to blob and the guide was not "
+                         "registered to the marketplace. ")
+            return False
+        else:
+            logger.info(f"Task registered successfully: \n {register}")
+
+        logger.info("Uploading Guide form to Blob storage container...")
+        blob_response =  Blob._guide_blob_upload(api_inputs=api_inputs, local_folder_path=local_folder_path,
+                                                 driver_id=task.id)
+
+        if blob_response == False:
+            logger.error("Failed to upload blob files for guide registration and hence Marketplace "
+                         "Registration was not completed. You will need to reregister before the guide will be shown in "
+                         "the Marketplace.")
+            return register, blob_response
+        else:
+            logger.info(f"Guide form successfully uploaded to blob. ")
+
+        # Request Clone of Assets to DataCenters
+        headers = api_inputs.api_headers.default
+
+        payload = {
+           "folderPath": task.id + '/'
+        }
+
+        url = f"{api_inputs.api_projects_endpoint}/{api_inputs.api_project_id}/guide/form/assets-clone"
+
+        logger.info("Sending request: POST %s", url)
+
+        blob_sync_response = requests.post(url, json=payload, headers=headers)
+        response_status = '{} {}'.format(blob_sync_response.status_code, blob_sync_response.reason)
+
+        logger.info(f"Sync Guide Form assets for DriverId={task.id} is {response_status}")
+
+        # Register to Marketplace
+        marketplace_response = add_marketplace_item(
+            api_inputs=api_inputs, name=task.marketplace_name, short_description=task.guide_short_description,
+            description=task.guide_description, version=task.guide_version, external_type=external_type,
+            external_id=task.id, tags=task.guide_tags, scope=scope, image_file_name=task.image_file_name,
+            card_image_file_name=task.card_image_file_name)
+
+        __, marketplace_df = (0, response) if isinstance(response, pandas.DataFrame) else marketplace_response
+
+        if __ != 0 and marketplace_df.shape[0] > 0:
+            logger.error(f"Marketplace registration was not successful. Please retry. Error response: \n {marketplace_df}")
+            return register, blob_response, marketplace_df
+        elif __ != 0 and marketplace_df.shape[0] == 0:
+            logger.error(f"Marketplace registration was not successful. Please retry. ")
+            return register, blob_response, __
+        else:
+            logger.info(f"Marketplace registration successful. \n {marketplace_df}")
+
+        return register, blob_response, blob_sync_response, marketplace_df
+
     @staticmethod
     def list_tasks(api_inputs: ApiInputs, search_name_pattern: str = '*'):
         """Get a list of the registered tasks.
@@ -524,7 +658,8 @@ class Automation:
                                   api_inputs: ApiInputs, data_feed_id: uuid.UUID,
                                   expected_delivery: EXPECTED_DELIVERY, email_subject_regex: str,
                                   email_address_domain: str, queue_name: QUEUE_NAME = 'task',
-                                  data_feed_name: str = None):
+                                  data_feed_name: str = None, task_priority: TASK_PRIORITY = 'standard',
+                                  task_framework: TASK_FRAMEWORK = 'TaskInsightsEngine'):
         """Deploy task as an email data feed.
 
         Deploys the created `task` as an email data feed. This allows the driver to ingest data sent via email. The
@@ -554,7 +689,13 @@ class Automation:
         data_feed_name : str, Optional
             The name of the data feed (to be displayed in Task Insights UI). If not provided, the API will automatically
             default to using the task.name property of the `task` passed to function.
-
+        task_priority: TASK_PRIORITY
+            Determines the priority of the datafeed tasks when processing. This equates to how much resources would be alloted
+            to run the task - 'default`, 'standard', or 'advanced'. Defaults to 'default'.
+        task_framework: TASK_FRAMEWORK
+            Determines the framework of the datafeed tasks when processing.
+            'PythonScriptFramework' for the old task runner engine.
+            'TaskInsightsEngine' for the new task running in container apps.
 
         Returns
         -------
@@ -600,6 +741,19 @@ class Automation:
                          'QUEUE_NAME literal: %s', QUEUE_NAME.__args__)
             return pandas.DataFrame()
 
+        if not set([task_framework]).issubset(set(TASK_FRAMEWORK.__args__)):
+            logger.error('task_framework parameter must be set to one of the allowed values defined by the '
+                         'TASK_FRAMEWORK literal: %s', TASK_FRAMEWORK.__args__)
+            return pandas.DataFrame()
+
+        if not set([task_priority]).issubset(set(TASK_PRIORITY.__args__)):
+            logger.error('task_priority parameter must be set to one of the allowed values defined by the '
+                         'TASK_PRIORITY literal: %s', TASK_PRIORITY.__args__)
+            return pandas.DataFrame()
+
+        logger.info('Task Framework is "%s"', str(task_framework))
+        logger.info('Task Priotiy is "%s"', str(task_priority))
+
         inbox_container = "data-exchange"
         payload = {
             "dataFeedId": str(data_feed_id),
@@ -609,6 +763,8 @@ class Automation:
             "expectedDelivery": expected_delivery,
             "sourceType": "Email",
             "queueName": queue_name,
+            "taskFramework": task_framework,
+            "taskPriority": task_priority,
             "email": {
                 "emailAddressDomain": email_address_domain,
                 "emailSubjectRegex": email_subject_regex,
@@ -640,7 +796,8 @@ class Automation:
                                             AnalyticsTask, LogicModuleTask, EventWorkOrderTask],
                                 api_inputs: ApiInputs, data_feed_id: uuid.UUID, expected_delivery: EXPECTED_DELIVERY,
                                 ftp_user_name: str, ftp_password: str, queue_name: QUEUE_NAME = 'task',
-                                data_feed_name: str = None):
+                                data_feed_name: str = None, task_priority: TASK_PRIORITY = 'standard',
+                                task_framework: TASK_FRAMEWORK = 'TaskInsightsEngine'):
         """Deploy the custom driver as an FTP data feed
 
         Deploys the custom driver to receive data via an FTP data feed. Sets the `ftp_user_name` & `ftp_password` and
@@ -666,6 +823,13 @@ class Automation:
         data_feed_name : str, Optional
             The name of the data feed (to be displayed in Task Insights UI). If not provided, will default to the
             task name.
+        task_priority: TASK_PRIORITY
+            Determines the priority of the datafeed tasks when processing. This equates to how much resources would be alloted
+            to run the task - 'default`, 'standard', or 'advanced'. Defaults to 'default'.
+        task_framework: TASK_FRAMEWORK
+            Determines the framework of the datafeed tasks when processing.
+            'PythonScriptFramework' for the old task runner engine.
+            'TaskInsightsEngine' for the new task running in container apps.
 
         Returns
         -------
@@ -697,8 +861,20 @@ class Automation:
                          'QUEUE_NAME literal: %s', QUEUE_NAME.__args__)
             return pandas.DataFrame()
 
+        if not set([task_framework]).issubset(set(TASK_FRAMEWORK.__args__)):
+            logger.error('task_framework parameter must be set to one of the allowed values defined by the '
+                         'TASK_FRAMEWORK literal: %s', TASK_FRAMEWORK.__args__)
+            return pandas.DataFrame()
+
+        if not set([task_priority]).issubset(set(TASK_PRIORITY.__args__)):
+            logger.error('task_priority parameter must be set to one of the allowed values defined by the '
+                         'TASK_PRIORITY literal: %s', TASK_PRIORITY.__args__)
+            return pandas.DataFrame()
+
         logger.info('Deploy %s (%s) as a data feed for ApiProjectID: %s', type(task).__name__, str(task.id),
                     api_inputs.api_project_id)
+        logger.info('Task Framework is "%s"', str(task_framework))
+        logger.info('Task Priotiy is "%s"', str(task_priority))
 
         inbox_container = "data-exchange"
         payload = {
@@ -709,6 +885,8 @@ class Automation:
             "expectedDelivery": expected_delivery,
             "sourceType": "Ftp",
             "queueName": queue_name,
+            "taskFramework": task_framework,
+            "taskPriority": task_priority,
             "ftp": {
                 "ftpUserName": ftp_user_name,
                 "ftpPassword": ftp_password,
@@ -739,7 +917,9 @@ class Automation:
     def deploy_as_upload_data_feed(task: Union[Task, IntegrationTask, DiscoverableIntegrationTask, QueueTask,
                                                AnalyticsTask, LogicModuleTask, EventWorkOrderTask],
                                    api_inputs: ApiInputs, data_feed_id: uuid.UUID, expected_delivery: EXPECTED_DELIVERY,
-                                   queue_name: QUEUE_NAME = 'task', data_feed_name: str = None):
+                                   queue_name: QUEUE_NAME = 'task', data_feed_name: str = None,
+                                   task_priority: TASK_PRIORITY = 'standard',
+                                   task_framework: TASK_FRAMEWORK = 'TaskInsightsEngine'):
         """Deploy the custom driver as a REST API end point Datafeed.
 
         To upload a file to the deployed data feed, use the UploadUrl from the response dataframe (with request type
@@ -774,6 +954,13 @@ class Automation:
         data_feed_name : str, Optional
             The name of the data feed (to be displayed in Task Insights UI). If not provided, will default to the
             task name.
+        task_priority: TASK_PRIORITY
+            Determines the priority of the datafeed tasks when processing. This equates to how much resources would be alloted
+            to run the task - 'default`, 'standard', or 'advanced'. Defaults to 'default'.
+        task_framework: TASK_FRAMEWORK
+            Determines the framework of the datafeed tasks when processing.
+            'PythonScriptFramework' for the old task runner engine.
+            'TaskInsightsEngine' for the new task running in container apps.
 
         Returns
         -------
@@ -805,8 +992,20 @@ class Automation:
                          'QUEUE_NAME literal: %s', QUEUE_NAME.__args__)
             return pandas.DataFrame()
 
+        if not set([task_framework]).issubset(set(TASK_FRAMEWORK.__args__)):
+            logger.error('task_framework parameter must be set to one of the allowed values defined by the '
+                         'TASK_FRAMEWORK literal: %s', TASK_FRAMEWORK.__args__)
+            return pandas.DataFrame()
+
+        if not set([task_priority]).issubset(set(TASK_PRIORITY.__args__)):
+            logger.error('task_priority parameter must be set to one of the allowed values defined by the '
+                         'TASK_PRIORITY literal: %s', TASK_PRIORITY.__args__)
+            return pandas.DataFrame()
+
         logger.info('Deploy %s (%s) as a data feed for ApiProjectID: %s', type(task).__name__, str(task.id),
                     api_inputs.api_project_id)
+        logger.info('Task Framework is "%s"', str(task_framework))
+        logger.info('Task Priotiy is "%s"', str(task_priority))
 
         payload = {
             "dataFeedId": str(data_feed_id),
@@ -816,6 +1015,8 @@ class Automation:
             "expectedDelivery": expected_delivery,
             "sourceType": "Upload",
             "queueName": queue_name,
+            "taskFramework": task_framework,
+            "taskPriority": task_priority,
             "upload": {
                 "placeholder": ""
             },
@@ -840,13 +1041,111 @@ class Automation:
 
         return df
 
+    @staticmethod
+    def deploy_as_blob_data_feed(task: BlobTask,
+                                 api_inputs: ApiInputs, data_feed_id: uuid.UUID, expected_delivery: EXPECTED_DELIVERY,
+                                 data_feed_name: str = None,
+                                 task_priority: TASK_PRIORITY = 'standard'):
+        """Deploy the BlobTask as a blob Datafeed.
+
+        Please Note: This task type requires external setup in Azure by Switch Automation Developers before a task can
+        be registered or deployed
+
+        Parameters
+        ----------
+        task : BlobTask
+            The custom driver class created from the Abstract Base Class 'BlobTask'
+        api_inputs : ApiInputs
+            Object returned by the initialize() function.
+        data_feed_id : uuid.UUID
+            The DataFeedId to update if existing, else will create a new record with the given value.
+        expected_delivery : EXPECTED_DELIVERY
+            The expected delivery frequency of the data.
+        data_feed_name : str, Optional
+            The name of the data feed (to be displayed in Task Insights UI). If not provided, will default to the
+            task name.
+        task_priority: TASK_PRIORITY
+            Determines the priority of the datafeed tasks when processing. This equates to how much resources would be
+            alloted to run the task - 'standard' or 'advanced'. Defaults to 'standard'.
+
+        Returns
+        -------
+        df : pandas.DataFrame
+            Dataframe containing the details of the deployed blob data feed.
+
+        """
+
+        if api_inputs.api_base_url == '' or api_inputs.bearer_token == '':
+            logger.error("You must call initialize() before using API.")
+            return pandas.DataFrame()
+
+        if not issubclass(task.__class__, BlobTask):
+            logger.error("Only task derived from BlobTask can be deployed as Blob Data Feed.")
+            return pandas.DataFrame()
+
+        if data_feed_id is None and data_feed_name is None:
+            data_feed_id = uuid.UUID('00000000-0000-0000-0000-000000000000')
+            data_feed_name = task.__class__.__qualname__
+        elif data_feed_id is not None and data_feed_name is None:
+            data_feed_id = data_feed_id
+            data_feed_name = data_feed_name
+
+        headers = api_inputs.api_headers.default
+
+        if not set([expected_delivery]).issubset(set(EXPECTED_DELIVERY.__args__)):
+            logger.error('expected_delivery parameter must be set to one of the allowed values defined by the '
+                         'EXPECTED_DELIVERY literal: %s', EXPECTED_DELIVERY.__args__)
+            return pandas.DataFrame()
+
+        if not set([task_priority]).issubset(set(TASK_PRIORITY.__args__).difference(set(["default"]))):
+            logger.error(f'task_priority parameter must be set to one of the allowed values defined by the '
+                         f'TASK_PRIORITY literal: {set(TASK_PRIORITY.__args__).difference(set(["default"]))}')
+            return pandas.DataFrame()
+
+        logger.info('Deploy %s (%s) as a data feed for ApiProjectID: %s', type(task).__name__, str(task.id),
+                    api_inputs.api_project_id)
+        logger.info('Task Framework is "%s"', 'TaskInsightsEngine')
+        logger.info('Task Priotiy is "%s"', str(task_priority))
+
+        payload = {
+            "dataFeedId": str(data_feed_id),
+            "driverId": str(task.id),
+            "name": data_feed_name,
+            "feedType": ",".join(task.mapping_entities),
+            "expectedDelivery": expected_delivery,
+            "sourceType": "Blob",
+            "queueName": "task",
+            "taskFramework": "TaskInsightsEngine",
+            "taskPriority": task_priority,
+        }
+
+        url = f"{api_inputs.api_projects_endpoint}/{api_inputs.api_project_id}/data-ingestion/tasks/deployment"
+
+        logger.info("Sending request: POST %s", url)
+
+        response = requests.post(url, json=payload, headers=headers)
+        response_status = '{} {}'.format(response.status_code, response.reason)
+        if response.status_code != 200:
+            logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
+                         response.reason)
+            return response_status, pandas.DataFrame()
+        elif len(response.text) == 0:
+            logger.error('No data returned for this API call. %s', response.request.url)
+            return response_status, pandas.DataFrame()
+
+        df = pandas.read_json(StringIO(response.text), typ='Series').to_frame().T
+        df.columns = _column_name_cap(df.columns)
+
+        return df
+
     @staticmethod
     def deploy_on_timer(task: Union[Task, IntegrationTask, DiscoverableIntegrationTask, QueueTask,
-                                    AnalyticsTask, LogicModuleTask, EventWorkOrderTask],
+                                    AnalyticsTask, LogicModuleTask, EventWorkOrderTask, IQTask],
                         api_inputs: ApiInputs, data_feed_id: uuid.UUID, expected_delivery: EXPECTED_DELIVERY,
                         cron_schedule: str, queue_name: QUEUE_NAME = "task", settings: dict = None,
                         schedule_timezone: SCHEDULE_TIMEZONE = 'Local', timezone_offset_minutes: int = None,
-                        data_feed_name: str = None):
+                        data_feed_name: str = None, task_priority: TASK_PRIORITY = 'standard',
+                        task_framework: TASK_FRAMEWORK = 'TaskInsightsEngine'):
         """Deploy driver to run on timer.
 
         Parameters
@@ -875,6 +1174,13 @@ class Automation:
         data_feed_name : str, Optional
             The name of the data feed (to be displayed in Task Insights UI). If not provided, will default to the
             task name.
+        task_priority: TASK_PRIORITY
+            Determines the priority of the datafeed tasks when processing. This equates to how much resources would be alloted
+            to run the task - 'default`, 'standard', or 'advanced'. Defaults to 'default'.
+        task_framework: TASK_FRAMEWORK
+            Determines the framework of the datafeed tasks when processing.
+            'PythonScriptFramework' for the old task runner engine.
+            'TaskInsightsEngine' for the new task running in container apps.
 
         Returns
         -------
@@ -946,9 +1252,21 @@ class Automation:
         #         if len(missing_required_keys) > 0 and len(missing_optional_keys) == 0:
         #             loggger.error(f"settings parameter is missing the following required key(s): {missing_required_keys}.")
 
+        if not set([task_framework]).issubset(set(TASK_FRAMEWORK.__args__)):
+            logger.error('task_framework parameter must be set to one of the allowed values defined by the '
+                         'TASK_FRAMEWORK literal: %s', TASK_FRAMEWORK.__args__)
+            return pandas.DataFrame()
+
+        if not set([task_priority]).issubset(set(TASK_PRIORITY.__args__)):
+            logger.error('task_priority parameter must be set to one of the allowed values defined by the '
+                         'TASK_PRIORITY literal: %s', TASK_PRIORITY.__args__)
+            return pandas.DataFrame()
+
         logger.info('Deploy %s (%s) on timer for ApiProjectID: %s and schedule: %s.', type(task).__name__,
                     str(task.id), api_inputs.api_project_id, cron_schedule)
         logger.info('Feed Type is %s', str(task.mapping_entities))
+        logger.info('Task Framework is "%s"', str(task_framework))
+        logger.info('Task Priotiy is "%s"', str(task_priority))
         logger.info('Settings to be passed to the driver on start are: %s', str(settings))
 
         payload = {
@@ -959,6 +1277,8 @@ class Automation:
             "expectedDelivery": expected_delivery,
             "sourceType": "Timer",
             "queueName": queue_name,
+            "taskFramework": task_framework,
+            "taskPriority": task_priority,
             "timer": {
                 "cronSchedule": cron_schedule,
                 "timezoneOffsetMinutes": timezone_offset_minutes,
@@ -1215,6 +1535,61 @@ class Automation:
 
         return df
 
+    @staticmethod
+    def run_data_feed(api_inputs: ApiInputs, data_feed_id: uuid.UUID, run_in_minutes: int = 0):
+        """Trigger an on-demand run of the python job based on data feed id. This will be sent to the queue for
+        processing and will undergo same procedure as the rest of the datafeed.
+
+        Parameters
+        ----------
+        api_inputs : ApiInputs
+            Object returned by initialize.initialize() function
+        data_feed_id : uuid.UUID
+            The unique identifier for the data feed that history should be retrieved for.
+        run_in_minutes : int
+            Optional parameter to delay running of datafeed in minutes. Defaults to 0.
+        """
+
+        if api_inputs.api_base_url == '' or api_inputs.bearer_token == '':
+            logger.error("You must call initialize() before using API.")
+            return pandas.DataFrame()
+
+        if data_feed_id is None or data_feed_id == '':
+            logger.error('The data_feed_id parameter must be provided. ')
+            return pandas.DataFrame()
+
+        headers = api_inputs.api_headers.default
+        url = f"{api_inputs.api_projects_endpoint}/{api_inputs.api_project_id}/data-ingestion/datafeed/{data_feed_id}/execute"
+        payload = {}
+        
+        if (run_in_minutes > 0):
+            url = f"{api_inputs.api_projects_endpoint}/{api_inputs.api_project_id}/data-ingestion/datafeed/{data_feed_id}/execute-in"
+            payload = {
+                "RunInMinutes": run_in_minutes
+            }
+
+        logger.info("Sending request: POST %s", url)
+
+        response = requests.request("POST", url, json=payload, timeout=20, headers=headers)
+
+        response_status = '{} {}'.format(response.status_code, response.reason)
+        if response.status_code != 200:
+            logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
+                         response.reason)
+            return response_status, pandas.DataFrame()
+        elif len(response.text) == 0:
+            logger.error('No data returned for this API call. %s', response.request.url)
+            return response_status, pandas.DataFrame()
+
+        # We need to wrap json results in StringIO when stringified JSON is present in the response.
+        # Symptoms of not doing this are: ValueError: Unexpected character found when decoding object value
+        # and incorrect suggestion in the error on installing a package called 'fsspec'
+        # Installing 'fsspec' package will not solve the issue.
+        df = pandas.read_json(StringIO(response.text),
+                              typ='Series').to_frame().T
+        df.columns = _column_name_cap(df.columns)
+        return df
+
     @staticmethod
     def data_feed_history_process_output(api_inputs: ApiInputs, data_feed_id: uuid.UUID = None,
                                          data_feed_file_status_id: uuid.UUID = None, row_number: int = None):
@@ -1406,7 +1781,8 @@ class Automation:
             logger.error('No data returned for this API call. %s', response.request.url)
             return response_status, pandas.DataFrame()
 
-        df = pandas.read_csv(StringIO(response.text), sep=",")  # , error_bad_lines=False)
+        # , error_bad_lines=False)
+        df = pandas.read_csv(StringIO(response.text), sep=",")
         return df
 
     @staticmethod
